/* This file was generated by syncqt. */
#ifndef QT_QTMULTIMEDIAQUICK_P_VERSION_H
#define QT_QTMULTIMEDIAQUICK_P_VERSION_H

#define QTMULTIMEDIAQUICK_P_VERSION_STR "5.6.2"

#define QTMULTIMEDIAQUICK_P_VERSION 0x050602

#endif // QT_QTMULTIMEDIAQUICK_P_VERSION_H
